{$I ussReader.inc}

unit ussReaderMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, DB, Grids, DBGrids, ComCtrls,
{$IFDEF USEADODATASET}
  ADODB,
{$ENDIF}
{$IFDEF USECLIENTDATASET}
  DbClient,
{$ENDIF}
  ussReaderTypes,
//  ussReaderIntf;
  ussReaderRun;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    Panel3: TPanel;
    Memo1: TMemo;
    ButtonDialogTunes: TButton;
    ButtonOpenConnection: TButton;
    ButtonCloseConnection: TButton;
    ButtonOpenQuery: TButton;
    ButtonCloseQuery: TButton;
    DBGrid1: TDBGrid;
    Splitter1: TSplitter;
    RichEdit1: TRichEdit;
    DataSource1: TDataSource;
    GroupBox1: TGroupBox;
    RadioButtonUseADODataSet: TRadioButton;
    RadioButtonUseClientDataSet: TRadioButton;
    CheckBoxFormatXML: TCheckBox;
    CheckBoxShowErrorDialog: TCheckBox;
    Label1: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure ButtonDialogTunesClick(Sender: TObject);
    procedure ButtonOpenConnectionClick(Sender: TObject);
    procedure ButtonCloseConnectionClick(Sender: TObject);
    procedure ButtonOpenQueryClick(Sender: TObject);
    procedure ButtonCloseQueryClick(Sender: TObject);
  private
    FVersion: Double;
    FConnectionActive: Boolean;
{$IFDEF USEADODATASET}
    FADOQuery: ADODB.TADOQuery;
{$ENDIF}
{$IFDEF USECLIENTDATASET}
    FClientDataSet: DBCLIENT.TClientDataSet;
{$ENDIF}
    FCommandText: string;
    FSaveCursor: TCursor;
    procedure DisableControls;
    procedure EnableControls;
  public
    procedure AddLog(const S: WideString; AInfoType: TMsgDlgType; ACustomColor: TColor = clBlack);
    procedure ShowError(const S: WideString);

    procedure OpenConnection;
    procedure CloseConnection;

    procedure OpenQuery1;
    procedure CloseQuery1;
  end;

var
  Form1: TForm1;

function CheckTunes: Boolean;

implementation

{$R *.dfm}

var
  GGroupName: string;

function CheckTunes: Boolean;
var
  xTunes: TUSSReaderDataTunes;
begin
  Result := False;

{$IFDEF XE6UP}
  SYSUTILS.FormatSettings.DateSeparator := '.';
  SYSUTILS.FormatSettings.ShortDateFormat := 'dd.MM.yyyy';
{$ELSE}
  SYSUTILS.DateSeparator := '.';
  SYSUTILS.ShortDateFormat := 'dd.MM.yyyy';
{$ENDIF}

//  GGroupName := ParamStr(1);
//  if GGroupName = '' then
//  repeat
//    if not DIALOGS.InputQuery('', '', GGroupName) then
//    begin
//      Application.Terminate;
//      Exit;
//    end;
//    GGroupName := SYSUTILS.Trim(GGroupName);
//  until GGroupName <> '';

//ussReader_SetTunesFile(Application.ExeName);
  ussReader_ClearTunes;
  ussReader_ReadTunes;

  ussReader_GetTunes(xTunes);
  GGroupName := xTunes[0].Common.ConnectionName;

  Result := True;
end;

//--- TForm1 -----------------------------------------------------------

procedure TForm1.FormCreate(Sender: TObject);
var
  S: ShortString;
begin
  Left := 8;
  Top := 8;

{$IFDEF USEADODATASET}
  FADOQuery := TADOQuery.Create(SELF);
{$ELSE}
  RadioButtonUseADODataSet.Enabled := False;
{$ENDIF}

{$IFDEF USECLIENTDATASET}
  FClientDataSet := TClientDataSet.Create(SELF);
{$ELSE}
  RadioButtonUseClientDataSet.Enabled := False;
{$ENDIF}

  Label1.Caption := '';

  Memo1.Lines.Clear;
  Memo1.Lines.Text := 'select GetDate()';

  RichEdit1.Lines.Clear;

  FVersion := ussReader_GetVersion;
  SYSTEM.Str(FVersion:3:1, S);
  Caption := 'ussReader ' + string(S);

  if (FVersion > 3.3) then
    ussReader_InitEx(Application.Handle, AddLog, ShowError)
  else
    ussReader_Init(AddLog);

  EnableControls;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  ussReader_Done;
end;

procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_F5) and (Shift = []) then
    ButtonOpenQueryClick(Sender);
end;

procedure TForm1.ButtonDialogTunesClick(Sender: TObject);
begin
  DisableControls;
  try
    ussReader_DialogTunes('', False);
  finally
    FConnectionActive := False;
    EnableControls;
  end;
end;

procedure TForm1.ButtonOpenConnectionClick(Sender: TObject);
begin
  OpenConnection;
end;

procedure TForm1.ButtonCloseConnectionClick(Sender: TObject);
begin
  CloseConnection;
end;

procedure TForm1.ButtonOpenQueryClick(Sender: TObject);
begin
  OpenQuery1;
end;

procedure TForm1.ButtonCloseQueryClick(Sender: TObject);
begin
  CloseQuery1;
end;

procedure TForm1.DisableControls;
begin
  FSaveCursor := Screen.Cursor;
  Screen.Cursor := crHourGlass;
  ButtonDialogTunes.Enabled := False;
  ButtonOpenConnection.Enabled := False;
  ButtonCloseConnection.Enabled := False;
  ButtonOpenQuery.Enabled := False;
  ButtonCloseQuery.Enabled := False;
end;

procedure TForm1.EnableControls;
begin
  ButtonDialogTunes.Enabled := True;
  ButtonOpenConnection.Enabled := True;
  ButtonCloseConnection.Enabled := FConnectionActive;
  ButtonOpenQuery.Enabled := FConnectionActive;
  ButtonCloseQuery.Enabled := FConnectionActive;
  Screen.Cursor := FSaveCursor;
end;

procedure TForm1.AddLog(const S: WideString; AInfoType: TMsgDlgType; ACustomColor: TColor = clBlack);
begin
  case AInfoType of
    mtInformation:
      begin
        RichEdit1.SelAttributes.Color := clBlack;
        RichEdit1.Lines.Add('I. ' + S);
      end;
    mtWarning:
      begin
        RichEdit1.SelAttributes.Color := clBlue;
        RichEdit1.Lines.Add('W. ' + S);
      end;
    mtError:
      begin
        RichEdit1.SelAttributes.Color := clRed;
        RichEdit1.Lines.Add('E. ' + S);
      end;
  else
    RichEdit1.SelAttributes.Color := ACustomColor;
    RichEdit1.Lines.Add(S);
  end;
  RichEdit1.Invalidate;
  RichEdit1.SetFocus;
  Application.ProcessMessages;
end;

procedure TForm1.ShowError(const S: WideString);
begin
  if CheckBoxShowErrorDialog.Checked then
    MessageDlg(S, mtError, [mbOk], 0);
end;

procedure TForm1.OpenConnection;
var
  xTunes: TUSSReaderDataTunes;
  Info: WideString;
begin
  FConnectionActive := False;

  ussReader_GetTunes(xTunes);
  GGroupName := xTunes[0].Common.ConnectionName;

  DisableControls;
  try
    AddLog('Open Connection', mtWarning);

    if (FVersion > 3.3) then
    begin
      if ussReader_OpenConnectionEx(Info, GGroupName, '') then;
    end
    else
      Info := ussReader_OpenConnection(GGroupName);

    if Info <> '' then // Ok
    begin
      FConnectionActive := True;
      AddLog(Info, mtInformation);
    end;
  finally
    EnableControls;
  end;
end;

procedure TForm1.CloseConnection;
begin
  DisableControls;
  try
    ussReader_CloseConnection(False);
    AddLog('Close Connection', mtWarning);
  finally
    FConnectionActive := False;
    EnableControls;
  end;
end;

procedure TForm1.OpenQuery1;
var
  T1, T2: TDateTime;
begin
  CloseQuery1;

  FCommandText := Memo1.Lines.Text;
  if SYSUTILS.Trim(FCommandText) = '' then Exit;

  DataSource1.DataSet := nil;

  DisableControls;
  try
{$IFDEF USEADODATASET}
    if RadioButtonUseADODataSet.Checked then
    begin
      AddLog('Open ADOQuery', mtWarning);
      T1 := Now;
      if ussReader_LoadADODataSet(FADOQuery, FCommandText, Integer(CheckBoxFormatXML.Checked)) then
      begin
        T2 := Now;
        DataSource1.DataSet := FADOQuery;
        Label1.Caption := TimeToStr(T2 - T1) + ' [Records ' + IntToStr(FADOQuery.RecordCount) + ']';
      end;
    end;
{$ENDIF}
{$IFDEF USECLIENTDATASET}
    if RadioButtonUseClientDataSet.Checked then
    begin
      AddLog('Open ClientDataSet', mtWarning);
      T1 := Now;
      if ussReader_LoadClientDataSet(FClientDataSet, FCommandText, Integer(CheckBoxFormatXML.Checked)) then
      begin
        T2 := Now;
        DataSource1.DataSet := FClientDataSet;
        Label1.Caption := TimeToStr(T2 - T1) + ' [Records ' + IntToStr(FClientDataSet.RecordCount) + ']';
      end;
    end;
{$ENDIF}
  finally
    EnableControls;
  end;
end;

procedure TForm1.CloseQuery1;
begin
{$IFDEF USEADODATASET}
  FADOQuery.Close;
{$ENDIF}
{$IFDEF USECLIENTDATASET}
  FClientDataSet.Close;
{$ENDIF}
  AddLog('Close Query', mtWarning);
  Label1.Caption := '';
end;

end.
