//******************************************************************************
//* uSBSS version 3.5
//* Module revision: 2017.07.07
//* www.rusbss.ru
//******************************************************************************

{$I ussReader.inc}

unit ussReaderTypes;

interface

uses
  Variants, Classes, Graphics,
{$IFDEF USEADODATASET}
  ADODB,
{$ENDIF}
{$IFDEF USECLIENTDATASET}
  DbClient,
{$ENDIF}
  Dialogs;

const
  ussReaderDLL = 'ussReader.dll';

  MaxHostNum = 10;
  MaxProxyNum = 2;

type
  TAddLogMethodW = procedure(const S: WideString;
    AInfoType: TMsgDlgType;
    ACustomColor: TColor = clBlack) of object;

  TShowErrorMethodW = procedure(const S: WideString) of object;

  Str40  = Array[0..40] of WideChar;
  Str128 = Array[0..128] of WideChar;
  Str255 = Array[0..255] of WideChar;

  TDataHost = packed record
    ComputerName: Str128;
    Port: Integer;
  end;

  TArrayHost = array[0..MaxHostNum] of TDataHost;

  TDataCommonRDM = packed record
    ConnectionName: Str128;
    TmpDir: Str255;
    RDMServerName: Str40;
    RDMServerGUID: Str40;
    UseTransaction: Boolean;
    ConnectionType: Integer; // 0-TSocketConnection 1-TWebConnection
  end;

  TDataSocket = packed record
    InterceptGUID: Str40;
    Hosts: TArrayHost;
  end;

  TDataWeb = packed record
    Proxy: Str128;
    ProxyByPass: Str128;
    UserName: Str40;
    Password: Str40;
    Hosts: TArrayHost;
  end;

  TDataRas = packed record
    PhoneBook: Str255;
    EntryName: Str128;
    UserName: Str40;
    Password: Str40;
    HangupBeforeDial: Boolean;
    UseRas: Boolean;
  end;

  TDataTuneMidas1 = packed record
    Common: TDataCommonRDM;
    Socket: TDataSocket;
    Web: TDataWeb;
    Ras: TDataRas;
    Use: Boolean;
  end;

  TUSSReaderDataTunes = array[0..MaxProxyNum] of TDataTuneMidas1;

//------------------------------------------------------------------------------

procedure ADODataSetFromRawData(ADataSet: TCustomADODataSet;
  AData: OleVariant;
  AFormat: TPersistFormat = pfADTG);

implementation

{$IFDEF USEADODATASET}
uses
{$IF CompilerVersion < 21}
  ActiveX, ComObj,
{$IFEND}
  ADOINT;

//------------------------------------------------------------------------------
{$IF CompilerVersion < 21}

// Constants for enum StreamTypeEnum
type
  StreamTypeEnum = TOleEnum;
  {$EXTERNALSYM StreamTypeEnum}
const
  adTypeBinary = $00000001;
  {$EXTERNALSYM adTypeBinary}
  adTypeText = $00000002;
  {$EXTERNALSYM adTypeText}

// Constants for enum StreamOpenOptionsEnum
type
  StreamOpenOptionsEnum = TOleEnum;
  {$EXTERNALSYM StreamOpenOptionsEnum}
const
  adOpenStreamUnspecified = $FFFFFFFF;
  {$EXTERNALSYM adOpenStreamUnspecified}
  adOpenStreamAsync = $00000001;
  {$EXTERNALSYM adOpenStreamAsync}
  adOpenStreamFromRecord = $00000004;
  {$EXTERNALSYM adOpenStreamFromRecord}

// Constants for enum StreamWriteEnum
type
  StreamWriteEnum = TOleEnum;
  {$EXTERNALSYM StreamWriteEnum}
const
  adWriteChar = $00000000;
  {$EXTERNALSYM adWriteChar}
  adWriteLine = $00000001;
  {$EXTERNALSYM adWriteLine}
  stWriteChar = $00000000;
  {$EXTERNALSYM stWriteChar}
  stWriteLine = $00000001;
  {$EXTERNALSYM stWriteLine}

// Constants for enum SaveOptionsEnum
type
  SaveOptionsEnum = TOleEnum;
  {$EXTERNALSYM SaveOptionsEnum}
const
  adSaveCreateNotExist = $00000001;
  {$EXTERNALSYM adSaveCreateNotExist}
  adSaveCreateOverWrite = $00000002;
  {$EXTERNALSYM adSaveCreateOverWrite}

// Constants for enum LineSeparatorEnum
type
  LineSeparatorEnum = TOleEnum;
  {$EXTERNALSYM LineSeparatorEnum}
const
  adLF = $0000000A;
  {$EXTERNALSYM adLF}
  adCR = $0000000D;
  {$EXTERNALSYM adCR}
  adCRLF = $FFFFFFFF;
  {$EXTERNALSYM adCRLF}

const
  CLASS_Stream: TGUID = '{00000566-0000-0010-8000-00AA006D2EA4}';

type
  ADO_LONGPTR = Integer;
// *********************************************************************//
// Interface: _Stream
// Flags:     (4432) Hidden Dual OleAutomation Dispatchable
// GUID:      {00000565-0000-0010-8000-00AA006D2EA4}
// *********************************************************************//
  _Stream = interface(IDispatch)
    ['{00000565-0000-0010-8000-00AA006D2EA4}']
    function Get_Size: ADO_LONGPTR; safecall;
    function Get_EOS: WordBool; safecall;
    function Get_Position: ADO_LONGPTR; safecall;
    procedure Set_Position(pPos: ADO_LONGPTR); safecall;
    function Get_Type_: StreamTypeEnum; safecall;
    procedure Set_Type_(ptype: StreamTypeEnum); safecall;
    function Get_LineSeparator: LineSeparatorEnum; safecall;
    procedure Set_LineSeparator(pLS: LineSeparatorEnum); safecall;
    function Get_State: ObjectStateEnum; safecall;
    function Get_Mode: ConnectModeEnum; safecall;
    procedure Set_Mode(pMode: ConnectModeEnum); safecall;
    function Get_Charset: WideString; safecall;
    procedure Set_Charset(const pbstrCharset: WideString); safecall;
    function Read(NumBytes: Integer): OleVariant; safecall;
    procedure Open(Source: OleVariant; Mode: ConnectModeEnum; Options: StreamOpenOptionsEnum;
                   const UserName: WideString; const Password: WideString); safecall;
    procedure Close; safecall;
    procedure SkipLine; safecall;
    procedure Write(Buffer: OleVariant); safecall;
    procedure SetEOS; safecall;
    procedure CopyTo(const DestStream: _Stream; CharNumber: ADO_LONGPTR); safecall;
    procedure Flush; safecall;
    procedure SaveToFile(const FileName: WideString; Options: SaveOptionsEnum); safecall;
    procedure LoadFromFile(const FileName: WideString); safecall;
    function ReadText(NumChars: Integer): WideString; safecall;
    procedure WriteText(const Data: WideString; Options: StreamWriteEnum); safecall;
    procedure Cancel; safecall;
    property Size: ADO_LONGPTR read Get_Size;
    property EOS: WordBool read Get_EOS;
    property Position: ADO_LONGPTR read Get_Position write Set_Position;
    property Type_: StreamTypeEnum read Get_Type_ write Set_Type_;
    property LineSeparator: LineSeparatorEnum read Get_LineSeparator write Set_LineSeparator;
    property State: ObjectStateEnum read Get_State;
    property Mode: ConnectModeEnum read Get_Mode write Set_Mode;
    property Charset: WideString read Get_Charset write Set_Charset;
  end;

  CoStream = class
    class function Create: _Stream;
  end;

class function CoStream.Create: _Stream;
begin
  Result := CreateComObject(CLASS_Stream) as _Stream;
end;

{$IFEND}
//------------------------------------------------------------------------------

type
  TCustomADODataSetPatch = class(TCustomADODataSet)
//protected
//  property CommandText;
//  property CommandType;
  end;

procedure ADODataSetFromStream(ADataSet: TCustomADODataSet;
  AStream: OleVariant);
var
  FRecordsetObject: ADOINT._Recordset;
begin
  ADataSet.Close;
  if (ADataSet is TADOQuery) then
    (ADataSet as TADOQuery).SQL.Clear
  else
    TCustomADODataSetPatch(ADataSet).CommandText := '';
  TCustomADODataSetPatch(ADataSet).CommandType := ADODB.cmdFile;
  ADataSet.LockType := ADODB.ltBatchOptimistic;

  FRecordsetObject := ADOINT.CoRecordset.Create;
  try
    FRecordsetObject.CursorLocation := ADOINT.adUseClient;
    FRecordsetObject.Open(AStream, VARIANTS.EmptyParam, adOpenUnspecified, adLockUnspecified, 0);
    ADataSet.Recordset := FRecordsetObject;
    ADataSet.Open;
  except
    FRecordsetObject := nil;
    raise;
  end;
end;

procedure ADODataSetFromRawData(ADataSet: TCustomADODataSet;
  AData: OleVariant;
  AFormat: TPersistFormat = pfADTG);
var
  ADOStream: _Stream;
begin
  ADOStream := CoStream.Create;
  try
    if AFormat = pfADTG then
      ADOStream.Type_ := adTypeBinary
    else
      ADOStream.Type_ := adTypeText;

    ADOStream.Open(
      VARIANTS.EmptyParam,     // Source: OleVariant;
      ADOINT.adModeUnknown,    // Mode: ConnectModeEnum;
      adOpenStreamUnspecified, // Options: StreamOpenOptionsEnum;
      '',                      // const UserName: WideString;
      '');                     // const Password: WideString);

    if AFormat = pfADTG then
      ADOStream.Write(AData)
    else
      ADOStream.WriteText(AData, stWriteLine);

    ADOStream.Position := 0;
    ADODataSetFromStream(ADataSet, ADOStream);
  finally
    ADOStream := nil;
  end;
end;
{$ENDIF}

end.

